"""
   Copyright 2009 Glenn Johnson

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import dbhash, shelve, datetime, sys, os, logging
import wx, wx.html, wx.calendar
from pkg_resources import resource_filename
from wx.lib.masked import NumCtrl
from wx.lib.wordwrap import wordwrap
from wx.lib.pubsub import Publisher
from details import *
from aim import *
from render import *

class App(wx.App):
    def OnInit(self):
	self.SetAppName('acclaim')
	self.CreateConfig()
	main_frame = Frame(self.config, self.db, self.cash, self.help)
	main_frame.Show()
	iconFile = resource_filename(__name__, 'icons/line_chart.ico')
	ib = wx.IconBundle()
	ib.AddIconFromFile(iconFile, wx.BITMAP_TYPE_ANY)
	wx.GetApp().GetTopWindow().SetIcons(ib)
	return True
    
    def CreateConfig(self):
	sp = wx.StandardPaths.Get()
	userData = sp.GetUserDataDir()
	if not os.path.isdir(userData):
	    os.mkdir(userData)
	
	self.config = wx.FileConfig(appName='acclaim',
		vendorName='Glenn Johnson',
		localFilename=os.path.join(userData, 'config'))
	
	logging.basicConfig(level=logging.INFO,
		format='%(asctime)s %(message)s',
		datefmt='%a, %d %b %Y %H:%M:%S',
		filename=os.path.join(userData, 'transaction.log'))
	
	self.db = shelve.open(os.path.join(userData, 'ACCLAIM.db'))
	
	#wx.FileSystem.AddHandler(wx.ZipFSHandler())
	helpFile = resource_filename(__name__, 'htmlhelp/ACCLAIMdoc.hhp')
	self.help = wx.html.HtmlHelpController(
		style=wx.html.HF_DEFAULT_STYLE ^ wx.html.HF_INDEX)
	self.help.SetTempDir(userData)
	self.help.UseConfig(self.config, rootpath='/Help')
	self.help.AddBook(helpFile)

	try:
	    self.cash = self.db['cash']

	except:
	    # need to set up initial account
	    self.cash = Cash()
	    self.cash.free = 0.00
	    self.cash.core = 50.00
	    self.cash.vix = 20.00
	    self.cash.level = 50.00
	    self.cash.interest_earned = 0.0
	    self.delay = 5
	    self.db['cash'] = self.cash
	    self.db['delay'] = self.delay
	    self.db['portfolios'] = {}
	    self.db.sync()

    def OnExit(self):
	self.config.Flush()
	self.db.close()

class NotEmptyValidator(wx.PyValidator):
    def __init__(self):
	wx.PyValidator.__init__(self)

    def Clone(self):
	return NotEmptyValidator()

    def Validate(self, win):
	textCtrl = self.GetWindow()
	text = textCtrl.GetValue()

	if len(text) == 0:
	    wx.MessageBox("This field must contain a value!", "Error",
		    wx.ICON_ERROR)
	    textCtrl.SetBackgroundColour("yellow")
	    textCtrl.SetFocus()
	    textCtrl.Refresh()
	    return False
	else:
	    textCtrl.SetBackgroundColour(
		    wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
	    textCtrl.Refresh()
	    return True

    def TransferToWindow(self):
	    return True

    def TransferFromWindow(self):
	    return True

class NewPortDialog(wx.Dialog):
    def __init__(self, mlabel):
	wx.Dialog.__init__(self, None, -1, "New Portfolio")
	vbox = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.BoxSizer(wx.VERTICAL)
	vbox.Add(vslot1, 1, wx.GROW)
	vslot2 = wx.FlexGridSizer(rows=3, cols=2, hgap=5, vgap=5)
	vbox.Add(vslot2)
	label = wx.StaticText(self, -1, 
		"Enter ticker symbol, description, and funding")
	vslot1.Add(label, 1, wx.ALIGN_CENTER | wx.BOTTOM, 20)
	label.SetForegroundColour("Blue")

	ticker_label = wx.StaticText(self, -1,
		"Enter ticker symbol")
	self.ticker_textctrl = wx.TextCtrl(self, -1,
		validator=NotEmptyValidator()) 
	descript_label = wx.StaticText(self, -1,
		"Enter description")
	self.descript_textctrl = wx.TextCtrl(self, -1, 
		size=(200,-1), validator=NotEmptyValidator())
	money_label = wx.StaticText(self, -1,
		mlabel)
	self.money_numctrl = NumCtrl(self, -1, 
		fractionWidth=2, selectOnEntry=False)

	vslot2.AddMany([ticker_label, self.ticker_textctrl, 
	    descript_label, self.descript_textctrl, 
	    money_label, self.money_numctrl])
	
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.ticker_textctrl.SetFocus()

class DepositWithdraw(wx.Dialog):
    def __init__(self, title, label):
	wx.Dialog.__init__(self, None, -1, title)
	vbox = wx.BoxSizer(wx.VERTICAL)
	money_label = wx.StaticText(self, -1,
		label)
	money_label.SetForegroundColour("Blue")
	self.money_numctrl = NumCtrl(self, -1, 
		fractionWidth=2, selectOnEntry=False)
	vbox.Add(money_label)
	vbox.Add(self.money_numctrl)
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.money_numctrl.SetFocus()

class BuySell(wx.Dialog):
    def __init__(self, title, symbol, price):
	wx.Dialog.__init__(self, None, -1, title)
	vbox = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.BoxSizer(wx.VERTICAL)
	vbox.Add(vslot1, 1, wx.GROW)
	vslot2 = wx.FlexGridSizer(rows=4, cols=2, hgap=5, vgap=5)
	vbox.Add(vslot2)
	label = wx.StaticText(self, -1, 
		"Transaction parameters for %s" % symbol.upper())
	vslot1.Add(label, 1, wx.ALIGN_CENTER | wx.BOTTOM, 20)
	label.SetForegroundColour("Blue")
	date_label = wx.StaticText(self, -1,
		"Transaction date ")
	self.cal = wx.calendar.CalendarCtrl(self, -1, wx.DateTime_Now(),
		style=wx.calendar.CAL_SEQUENTIAL_MONTH_SELECTION)
	vslot2.Add(date_label)
	vslot2.Add(self.cal)
	
	shares_label = wx.StaticText(self, -1,
		"Number of shares ")
	self.shares_numctrl = NumCtrl(self, -1, 
		fractionWidth=6, selectOnEntry=False)
	vslot2.Add(shares_label)
	vslot2.Add(self.shares_numctrl)
	
	price_label = wx.StaticText(self, -1,
		"Price per share ")
	self.price_numctrl = NumCtrl(self, -1, 
		value=price, fractionWidth=6, selectOnEntry=True)
	vslot2.Add(price_label)
	vslot2.Add(self.price_numctrl)

	commision_label = wx.StaticText(self, -1,
		"Commission ")
	self.commission_numctrl = NumCtrl(self, -1, 
		fractionWidth=2, selectOnEntry=False)
	vslot2.Add(commision_label)
	vslot2.Add(self.commission_numctrl)
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 0, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.shares_numctrl.SetFocus()

class Frame(wx.Frame):
    def __init__(self, config, db, cash, help):
	self.config = config
	self.db = db
	self.cash = cash
	self.help = help
	self.config.SetPath('/Frames')
	width = self.config.ReadInt('action_width', 700)
	height = self.config.ReadInt('action_height', 600)
	wx.Frame.__init__(self, None, -1,
		"Adjusted Core Cash Level Automatic Investment Management",
		size=(width, height))
	self.panel = wx.Panel(self, -1)

	self.recs_html = MyHtmlWindow(self.panel, -1)
	if "gtk2" in wx.PlatformInfo:
	    self.recs_html.SetStandardFonts()
	self.recs_html.SetRelatedFrame(self, self.GetTitle() + ": %s")
	self.recs_html.SetRelatedStatusBar(0)
	self.SetStatusBarPane(1)

	sizer = wx.BoxSizer(wx.VERTICAL)
	sizer.Add(self.recs_html, 1, wx.GROW)
	
	subsizer = wx.FlexGridSizer(rows=1, cols=7, hgap=5, vgap=5)
	sizer.Add(subsizer)
	
	sc_cash_label = wx.StaticText(self.panel, -1, "Cash Level (%)")
	self.sc_cash = wx.SpinCtrl(self.panel, -1, 
		initial=self.cash.core, min=20, max=80)
	self.Bind(wx.EVT_SPINCTRL, self.OnCashLevel, self.sc_cash)

	sc_delay_label = wx.StaticText(self.panel, -1, "Buy delay (days)")
	self.sc_delay = wx.SpinCtrl(self.panel, -1, initial=self.db['delay'])
	self.Bind(wx.EVT_SPINCTRL, self.OnDelay, self.sc_delay)

	yahoo_btn = wx.Button(self.panel, -1, "&Update")
	self.Bind(wx.EVT_BUTTON, self.OnUpdate, yahoo_btn)
	
	port_btn = wx.Button(self.panel, -1, "De&tails")
	self.Bind(wx.EVT_BUTTON, 
		lambda event: self.OnDetails(event, self.recs_html),
		port_btn)
	subsizer.AddMany([
	    (sc_cash_label, 0, wx.ALIGN_CENTER |wx.LEFT, 20),
	    (self.sc_cash, 0, wx.ALIGN_CENTER | wx.RIGHT, 20),
	    (sc_delay_label, 0, wx.ALIGN_CENTER),
	    (self.sc_delay, 0, wx.ALIGN_CENTER | wx.RIGHT, 20),
	    ((25, -1)),
	    (port_btn, 0, wx.ALIGN_CENTER),
	    (yahoo_btn, 0, wx.ALIGN_CENTER),
	    ])
	self.panel.SetSizer(sizer)
	self.panel.Layout()
	self.SetMinSize(sizer.GetMinSize())
	self.recs_html.SetFocus()
	
	self.render = Render()
	self.render.recs(self.recs_html, self.db)

	self.statusbar = self.CreateStatusBar(number=2)
	self.statusbar.SetStatusText("Adjusted cash level (%%): %.2f" 
		% self.cash.level, 0)

	menubar = wx.MenuBar()

	file_menu = wx.Menu()
	exit = file_menu.Append(wx.ID_EXIT)
	self.Bind(wx.EVT_MENU, self.OnExit, exit)
	menubar.Append(file_menu, "&File")

	# Cash menu
	cash_menu = wx.Menu()
	deposit = cash_menu.Append(-1, "&Deposit...",
		"Deposit cash into account")
	self.Bind(wx.EVT_MENU, self.OnDeposit, deposit)

	withdraw = cash_menu.Append(-1, "&Withdraw...",
		"Withdraw cash from account")
	self.Bind(wx.EVT_MENU, self.OnWithdraw, withdraw)
	fund_port = cash_menu.Append(-1, "&Fund portfolio...",
		"Deposit funds into a portfolio")
	self.Bind(wx.EVT_MENU, self.OnFund, fund_port)
	siphon_port = cash_menu.Append(-1, "Si&phon Funds...",
		"Siphon funds from a portfolio")
	self.Bind(wx.EVT_MENU, self.OnSiphon, siphon_port)
	menubar.Append(cash_menu, "&Cash")

	# Portfolio menu
	port_menu = wx.Menu()
	
	new_port = port_menu.Append(wx.ID_NEW,
		help="Create a new portfolio")
	self.Bind(wx.EVT_MENU, self.OnNewport, new_port)
	buy = port_menu.Append(-1, "&Buy stock...",
		"Buy shares of portfolio stock")
	self.Bind(wx.EVT_MENU, self.OnBuy, buy)
	sell = port_menu.Append(-1, "&Sell stock...",
		"Sell shares of portfolio stock")
	self.Bind(wx.EVT_MENU, self.OnSell, sell)
	delete = port_menu.Append(-1, "&Delete",
		"Delete portfolio")
	self.Bind(wx.EVT_MENU, self.OnDelete, delete)
	detail_menu = port_menu.Append(-1, "De&tails",
		"View details of portfolios")
	self.Bind(wx.EVT_MENU, 
		lambda event: self.OnDetails(event, self.recs_html), 
		detail_menu)
	yahoo = port_menu.Append(-1, "&Update prices",
		"Update prices via Yahoo!")
	self.Bind(wx.EVT_MENU, self.OnUpdate, yahoo)
	
	menubar.Append(port_menu, "&Portfolio")

	# Distributions menu
	dist_menu = wx.Menu()
	cap_gain = dist_menu.Append(-1, "Capital &Gains...",
		"Capital Gains distribution")
	dividend = dist_menu.Append(-1, "&Dividends...",
		"Dividend distribution")
	interest = dist_menu.Append(-1, "&Interest...",
		"Interest on cash")
	self.Bind(wx.EVT_MENU, self.OnCapGains, cap_gain)
	self.Bind(wx.EVT_MENU, self.OnDividend, dividend)
	self.Bind(wx.EVT_MENU, self.OnInterest, interest)

	menubar.Append(dist_menu, "&Distributions")

	# Help
	help_menu = wx.Menu()
	help = help_menu.Append(wx.ID_HELP,
		help="Help system")
	self.Bind(wx.EVT_MENU, self.OnHelp, help)
	ex_help = help_menu.Append(-1, "Help (&External viewer)",
		"View help in Web Browser")
	self.Bind(wx.EVT_MENU, self.OnExHelp, ex_help)
	about = help_menu.Append(wx.ID_ABOUT)
	self.Bind(wx.EVT_MENU, self.OnAbout, about)
	menubar.Append(help_menu, "&Help")

	self.SetMenuBar(menubar)
	Publisher().subscribe(self.OnDetailsMessage, 'details.size')

    def OnDetailsMessage(self, message):
	width, height = message.data
	self.config.SetPath('/Frames')
	self.config.Write('detail_width', str(width))
	self.config.Write('detail_height', str(height))

    def OnHelp(self, event):
	#wx.MessageBox('Help files not completed yet.', parent=self)
	self.help.DisplayContents()

    def OnExHelp(self, event):
	htmlFile = resource_filename(__name__, 'html/index.html')
	wx.LaunchDefaultBrowser(htmlFile)

    def OnAbout(self, event):
	info = wx.AboutDialogInfo()
	info.Name = "ACCLAIM"
	info.Version = "0.1.1"
	info.Copyright = "Copyright (C) 2009 Glenn Johnson"
	info.Description = wordwrap(
		"ACCLAIM "
		"(Adjusted Core Cash Level Automatic Investment Management) "
		"is a program derived from Robert Lichello's AIM method. "
		"It attempts to be more responsive to market conditions "
		"and obtain a more controlled cash-to-equity ratio compared "
		"to conventional AIM."
		"\n\n"
		"ACCLAIM is written in Python and uses the wxPython GUI "
		"toolkit.",
		400, wx.ClientDC(self))
	info.Developers = ['Glenn Johnson <glennpj@gmail.com>']
	info.DocWriters = ['Glenn Johnson <glennpj@gmail.com>']
	info.Artists = [
		'VisualPharm http://www.visualpharm.com/finance_icon_set/']
	if "gtk2" in wx.PlatformInfo:
	    info.License = wordwrap("""
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this program except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
   """,
	    800, wx.ClientDC(self))
	    info.WebSite = ("http://bitbucket.org/glennpj/acclaim/overview", 
		    "ACCLAIM Web Site")
	wx.AboutBox(info)

    def OnCapGains(self, event):
	D = self.db['portfolios']
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = D[symbol]
	label = "%s Capital Gains distribution" % symbol.upper()
	money_d = DepositWithdraw('Capital Gains', label)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    money_d.Destroy()
	    stock.capital_gain(amount)
	    stock.calc(self.cash.level, self.db['delay'])
	    self.db['portfolios'] = D
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    money_d.Destroy()

    def OnDividend(self, event):
	D = self.db['portfolios']
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = D[symbol]
	label = "%s Dividend distribution" % symbol.upper()
	money_d = DepositWithdraw('Dividends', label)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    money_d.Destroy()
	    stock.dividend(amount)
	    stock.calc(self.cash.level, self.db['delay'])
	    self.db['portfolios'] = D
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    money_d.Destroy()

    def OnInterest(self, event):
	label = "Interest on cash earned"
	money_d = DepositWithdraw('Interest on cash', label)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    self.cash.interest(amount)
	    money_d.Destroy()
	    self.db['cash'] = self.cash
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    money_d.Destroy()

    def OnDelete(self, event):
	D = self.db['portfolios']
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = D[symbol]
	if stock.shares:
	    wx.MessageBox("You must sell all shares first", parent=self)
	    return
	self.cash.deposit(stock.cash)
	D.pop(symbol)
	self.db['portfolios'] = D
	self.db['cash'] = self.cash
	self.db.sync()
	self.render.recs(self.recs_html, self.db)

    def OnSell(self, event):
	D = self.db['portfolios']
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = D[symbol]
	sell_d = BuySell('Sell', symbol, stock.price)
	sell_d.shares_numctrl.SetMax(stock.shares)
	sell_d.shares_numctrl.SetLimited(True)
	result = sell_d.ShowModal()
	if result == wx.ID_OK:
	    date = sell_d.cal.GetDate()
	    shares = sell_d.shares_numctrl.GetValue()
	    price = sell_d.price_numctrl.GetValue()
	    commission = sell_d.commission_numctrl.GetValue()
	    sell_d.Destroy()
	    stock.sell(date, shares, price, commission)
	    stock.calc(self.cash.level, self.db['delay'])
	    self.db['portfolios'] = D
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	else:
	    sell_d.Destroy()

    def OnBuy(self, event):
	D = self.db['portfolios']
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = D[symbol]
	buy_d = BuySell('Buy', symbol, stock.price)
	result = buy_d.ShowModal()
	if result == wx.ID_OK:
	    date = buy_d.cal.GetDate()
	    shares = buy_d.shares_numctrl.GetValue()
	    price = buy_d.price_numctrl.GetValue()
	    commission = buy_d.commission_numctrl.GetValue()
	    buy_d.Destroy()
	    stock.buy(date, shares, price, commission)
	    stock.calc(self.cash.level, self.db['delay'])
	    self.db['portfolios'] = D
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	else:
	    buy_d.Destroy()

    def OnSiphon(self, event):
	D = self.db['portfolios']
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = D[symbol]
	label = "Enter amount to take %s\n($%.2f available)" % (
		symbol.upper(), stock.cash)
	money_d = DepositWithdraw('Siphon Cash', label)
	money_d.money_numctrl.SetMax(stock.cash)
	money_d.money_numctrl.SetLimited(True)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    self.cash.deposit(amount)
	    money_d.Destroy()
	    stock.cash_out += amount
	    stock.cash -= amount
	    stock.calc(self.cash.level, self.db['delay'])
	    self.db['portfolios'] = D
	    self.db['cash'] = self.cash
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    money_d.Destroy()

    def OnFund(self, event):
	D = self.db['portfolios']
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = D[symbol]
	label = "Enter amount to fund %s\n($%.2f available)" % (
		symbol.upper(), self.cash.free)
	money_d = DepositWithdraw('Transfer Cash', label)
	money_d.money_numctrl.SetMax(self.cash.free)
	money_d.money_numctrl.SetLimited(True)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    self.cash.withdraw(amount)
	    money_d.Destroy()
	    stock.cash_in += amount
	    stock.cash += amount
	    stock.calc(self.cash.level, self.db['delay'])
	    self.db['portfolios'] = D
	    self.db['cash'] = self.cash
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    money_d.Destroy()

    def OnNewport(self, event):
	D = self.db['portfolios']
	stock = Stock()
	mlabel = "Initial funding\n($%.2f available)" % self.cash.free
	newport_d = NewPortDialog(mlabel)
	newport_d.money_numctrl.SetMax(self.cash.free)
	newport_d.money_numctrl.SetLimited(True)
	result = newport_d.ShowModal()
	if result == wx.ID_OK:
	    stock.symbol = newport_d.ticker_textctrl.GetValue()
	    stock.description = newport_d.descript_textctrl.GetValue()
	    amount = newport_d.money_numctrl.GetValue()
	    newport_d.Destroy()
	    self.cash.withdraw(amount)
	    stock.new()
	    stock.cash_in += amount
	    stock.cash += amount
	    stock.calc(self.cash.level, self.db['delay'])
	    D[stock.symbol] = stock
	    self.db['portfolios'] = D
	    self.db['cash'] = self.cash
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    newport_d.Destroy()
	    return

    def OnUpdate(self, event):
	self.cash.get_vix()
	self.cash.transform_vix()
	self.statusbar.SetStatusText("Adjusted cash level (%%): %.2f" 
		% self.cash.level, 0)
	self.db['cash']=self.cash
	self.db.sync()
	self.sc_cash.SetValue(self.cash.core)
	D = self.db['portfolios']
	list = sorted(D.keys())
	tickers = len(list)
	dialog = wx.ProgressDialog("Updating prices",
		"",  tickers, parent=self,
		style=wx.PD_CAN_ABORT | wx.PD_AUTO_HIDE)
	keepGoing = True
	count = 0
	while keepGoing and count < tickers:
	    ticker = list[count]
	    stock = D[ticker]
	    stock.yahoo()
	    stock.calc(self.cash.level, self.db['delay'])
	    D[ticker] = stock
	    count += 1
	    (keepGoing,skip)= dialog.Update(
		    count, "updated %s" % ticker.upper())
	dialog.Destroy()
	self.db['portfolios'] = D
	self.db.sync()
	self.render.recs(self.recs_html, self.db)
	Publisher().sendMessage('update.prices', self.db)

    def OnDetails(self, event, recs_html):
	try:
	    detailsFrame = DetailFrame(self.db, recs_html, self.config)
	    detailsFrame.Show()
	except:
	    wx.MessageBox("No portfolios set up yet", parent=self) 

    def OnCashLevel(self, event):
	self.cash.core = float(self.sc_cash.GetValue())
	self.cash.transform_vix()
	self.statusbar.SetStatusText("Adjusted cash level (%%): %.2f" 
		% self.cash.level, 0)
	self.db['cash'] = self.cash
	self.db.sync()
	D = self.db['portfolios']
	for ticker in sorted(D.keys()):
	    stock = D[ticker]
	    # need to save portfolio control values because they may change
	    # while cycling through cash level values
	    buy_pc = stock.buy_portfolio_control
	    sell_pc = stock.sell_portfolio_control
	    stock.calc(self.cash.level, self.db['delay'])
	    stock.buy_portfolio_control = buy_pc
	    stock.sell_portfolio_control = sell_pc
	    D[ticker] = stock
	self.db['portfolios'] = D
	self.db.sync()
	self.render.recs(self.recs_html, self.db)
	Publisher().sendMessage('update.cash_level', self.db)

    def OnDelay(self, event):
	self.delay = self.sc_delay.GetValue()
	self.db['delay'] = int(self.delay)
	self.db['cash'] = self.cash
	self.db.sync()
	D = self.db['portfolios']
	for ticker in sorted(D.keys()):
	    stock = D[ticker]
	    stock.calc(self.cash.level, self.db['delay'])
	    D[ticker] = stock
	self.db['portfolios'] = D
	self.db.sync()
	self.render.recs(self.recs_html, self.db)

    def OnExit(self, event):
	width, height = self.GetSize()
	self.config.SetPath('/Frames')
	self.config.Write('action_width', str(width))
	self.config.Write('action_height', str(height))
	for win in wx.GetTopLevelWindows():
	    win.Close()

    def OnDeposit(self, event):
	label = "Enter amount to deposit\n($%.2f balance)" % self.cash.free
	money_d = DepositWithdraw('Deposit Cash', label)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    self.cash.deposit(amount)
	    money_d.Destroy()
	    self.db['cash'] = self.cash
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    money_d.Destroy()

    def OnWithdraw(self, event):
	label = "Enter amount to withdraw\n($%.2f available)" % self.cash.free
	money_d = DepositWithdraw('Withdraw Cash', label)
	money_d.money_numctrl.SetMax(self.cash.free)
	money_d.money_numctrl.SetLimited(True)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    self.cash.withdraw(amount)
	    money_d.Destroy()
	    self.db['cash'] = self.cash
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db)
	    return
	else:
	    money_d.Destroy()

def main():
    app = App(0)
    app.MainLoop()

if __name__ == "__main__":
    main()
